@@ -38,97 +38,10 @@ the use of this software, even if advised of the possibility of such damage.
 
 
 #include "test_precomp.hpp"
+#include "test_aruco_utils.hpp"
 
 namespace opencv_test { namespace {
 
-static double deg2rad(double deg) { return deg * CV_PI / 180.; }
-
-/**
- * @brief Get rvec and tvec from yaw, pitch and distance
- */
-static void getSyntheticRT(double yaw, double pitch, double distance, Mat &rvec, Mat &tvec) {
-
-    rvec = Mat(3, 1, CV_64FC1);
-    tvec = Mat(3, 1, CV_64FC1);
-
-    // Rvec
-    // first put the Z axis aiming to -X (like the camera axis system)
-    Mat rotZ(3, 1, CV_64FC1);
-    rotZ.ptr< double >(0)[0] = 0;
-    rotZ.ptr< double >(0)[1] = 0;
-    rotZ.ptr< double >(0)[2] = -0.5 * CV_PI;
-
-    Mat rotX(3, 1, CV_64FC1);
-    rotX.ptr< double >(0)[0] = 0.5 * CV_PI;
-    rotX.ptr< double >(0)[1] = 0;
-    rotX.ptr< double >(0)[2] = 0;
-
-    Mat camRvec, camTvec;
-    composeRT(rotZ, Mat(3, 1, CV_64FC1, Scalar::all(0)), rotX, Mat(3, 1, CV_64FC1, Scalar::all(0)),
-              camRvec, camTvec);
-
-    // now pitch and yaw angles
-    Mat rotPitch(3, 1, CV_64FC1);
-    rotPitch.ptr< double >(0)[0] = 0;
-    rotPitch.ptr< double >(0)[1] = pitch;
-    rotPitch.ptr< double >(0)[2] = 0;
-
-    Mat rotYaw(3, 1, CV_64FC1);
-    rotYaw.ptr< double >(0)[0] = yaw;
-    rotYaw.ptr< double >(0)[1] = 0;
-    rotYaw.ptr< double >(0)[2] = 0;
-
-    composeRT(rotPitch, Mat(3, 1, CV_64FC1, Scalar::all(0)), rotYaw,
-              Mat(3, 1, CV_64FC1, Scalar::all(0)), rvec, tvec);
-
-    // compose both rotations
-    composeRT(camRvec, Mat(3, 1, CV_64FC1, Scalar::all(0)), rvec,
-              Mat(3, 1, CV_64FC1, Scalar::all(0)), rvec, tvec);
-
-    // Tvec, just move in z (camera) direction the specific distance
-    tvec.ptr< double >(0)[0] = 0.;
-    tvec.ptr< double >(0)[1] = 0.;
-    tvec.ptr< double >(0)[2] = distance;
-}
-
-/**
- * @brief Project a synthetic marker
- */
-static void projectMarker(Mat &img, Ptr<aruco::Dictionary> dictionary, int id,
-                          vector< Point3f > markerObjPoints, Mat cameraMatrix, Mat rvec, Mat tvec,
-                          int markerBorder) {
-
-
-    Mat markerImg;
-    const int markerSizePixels = 100;
-    aruco::drawMarker(dictionary, id, markerSizePixels, markerImg, markerBorder);
-
-    Mat distCoeffs(5, 1, CV_64FC1, Scalar::all(0));
-    vector< Point2f > corners;
-    projectPoints(markerObjPoints, rvec, tvec, cameraMatrix, distCoeffs, corners);
-
-    vector< Point2f > originalCorners;
-    originalCorners.push_back(Point2f(0, 0));
-    originalCorners.push_back(Point2f((float)markerSizePixels, 0));
-    originalCorners.push_back(Point2f((float)markerSizePixels, (float)markerSizePixels));
-    originalCorners.push_back(Point2f(0, (float)markerSizePixels));
-
-    Mat transformation = getPerspectiveTransform(originalCorners, corners);
-
-    Mat aux;
-    const char borderValue = 127;
-    warpPerspective(markerImg, aux, transformation, img.size(), INTER_NEAREST, BORDER_CONSTANT,
-                    Scalar::all(borderValue));
-
-    // copy only not-border pixels
-    for(int y = 0; y < aux.rows; y++) {
-        for(int x = 0; x < aux.cols; x++) {
-            if(aux.at< unsigned char >(y, x) == borderValue) continue;
-            img.at< unsigned char >(y, x) = aux.at< unsigned char >(y, x);
-        }
-    }
-}
-
 /**
  * @brief Get a synthetic image of Chessboard in perspective
  */
@@ -145,7 +58,7 @@ static Mat projectChessboard(int squaresX, int squaresY, float squareSize, Size
             float startX = float(x) * squareSize;
 
             vector< Point3f > squareCorners;
-            squareCorners.push_back(Point3f(startX, startY, 0));
+            squareCorners.push_back(Point3f(startX, startY, 0) - Point3f(squaresX*squareSize/2.f, squaresY*squareSize/2.f, 0.f));
             squareCorners.push_back(squareCorners[0] + Point3f(squareSize, 0, 0));
             squareCorners.push_back(squareCorners[0] + Point3f(squareSize, squareSize, 0));
             squareCorners.push_back(squareCorners[0] + Point3f(0, squareSize, 0));
@@ -180,8 +93,8 @@ static Mat projectCharucoBoard(Ptr<aruco::CharucoBoard> &board, Mat cameraMatrix
 
     // project markers
     Mat img = Mat(imageSize, CV_8UC1, Scalar::all(255));
-    for(unsigned int m = 0; m < board->ids.size(); m++) {
-        projectMarker(img, board->dictionary, board->ids[m], board->objPoints[m], cameraMatrix, rvec,
+    for(unsigned int indexMarker = 0; indexMarker < board->ids.size(); indexMarker++) {
+        projectMarker(img, board.staticCast<aruco::Board>(), indexMarker, cameraMatrix, rvec,
                       tvec, markerBorder);
     }
 
@@ -222,7 +135,7 @@ void CV_CharucoDetection::run(int) {
     Ptr<aruco::Dictionary> dictionary = aruco::getPredefinedDictionary(aruco::DICT_6X6_250);
     Ptr<aruco::CharucoBoard> board = aruco::CharucoBoard::create(4, 4, 0.03f, 0.015f, dictionary);
 
-    cameraMatrix.at< double >(0, 0) = cameraMatrix.at< double >(1, 1) = 650;
+    cameraMatrix.at< double >(0, 0) = cameraMatrix.at< double >(1, 1) = 600;
     cameraMatrix.at< double >(0, 2) = imgSize.width / 2;
     cameraMatrix.at< double >(1, 2) = imgSize.height / 2;
 
@@ -230,15 +143,15 @@ void CV_CharucoDetection::run(int) {
 
     // for different perspectives
     for(double distance = 0.2; distance <= 0.4; distance += 0.2) {
-        for(int yaw = 0; yaw < 360; yaw += 100) {
-            for(int pitch = 30; pitch <= 90; pitch += 50) {
+        for(int yaw = -55; yaw <= 50; yaw += 25) {
+            for(int pitch = -55; pitch <= 50; pitch += 25) {
 
                 int markerBorder = iter % 2 + 1;
                 iter++;
 
                 // create synthetic image
                 Mat rvec, tvec;
-                Mat img = projectCharucoBoard(board, cameraMatrix, deg2rad(pitch), deg2rad(yaw),
+                Mat img = projectCharucoBoard(board, cameraMatrix, deg2rad(yaw), deg2rad(pitch),
                                               distance, imgSize, markerBorder, rvec, tvec);
 
                 // detect markers
@@ -269,7 +182,13 @@ void CV_CharucoDetection::run(int) {
 
                 // check results
                 vector< Point2f > projectedCharucoCorners;
-                projectPoints(board->chessboardCorners, rvec, tvec, cameraMatrix, distCoeffs,
+
+                // copy chessboardCorners
+                vector<Point3f> copyChessboardCorners = board->chessboardCorners;
+                // move copyChessboardCorners points
+                for (size_t i = 0; i < copyChessboardCorners.size(); i++)
+                    copyChessboardCorners[i] -= board->rightBottomBorder / 2.f;
+                projectPoints(copyChessboardCorners, rvec, tvec, cameraMatrix, distCoeffs,
                               projectedCharucoCorners);
 
                 for(unsigned int i = 0; i < charucoIds.size(); i++) {
@@ -327,16 +246,16 @@ void CV_CharucoPoseEstimation::run(int) {
 
     Mat distCoeffs(5, 1, CV_64FC1, Scalar::all(0));
     // for different perspectives
-    for(double distance = 0.2; distance <= 0.4; distance += 0.2) {
-        for(int yaw = 0; yaw < 360; yaw += 100) {
-            for(int pitch = 30; pitch <= 90; pitch += 50) {
+    for(double distance = 0.2; distance <= 0.3; distance += 0.1) {
+        for(int yaw = -55; yaw <= 50; yaw += 25) {
+            for(int pitch = -55; pitch <= 50; pitch += 25) {
 
                 int markerBorder = iter % 2 + 1;
                 iter++;
 
                 // get synthetic image
                 Mat rvec, tvec;
-                Mat img = projectCharucoBoard(board, cameraMatrix, deg2rad(pitch), deg2rad(yaw),
+                Mat img = projectCharucoBoard(board, cameraMatrix, deg2rad(yaw), deg2rad(pitch),
                                               distance, imgSize, markerBorder, rvec, tvec);
 
                 // detect markers
@@ -347,11 +266,7 @@ void CV_CharucoPoseEstimation::run(int) {
                 params->markerBorderBits = markerBorder;
                 aruco::detectMarkers(img, dictionary, corners, ids, params);
 
-                if(ids.size() == 0) {
-                    ts->printf(cvtest::TS::LOG, "Marker detection failed");
-                    ts->set_failed_test_info(cvtest::TS::FAIL_MISMATCH);
-                    return;
-                }
+                ASSERT_EQ(ids.size(), board->ids.size());
 
                 // interpolate charuco corners
                 vector< Point2f > charucoCorners;
@@ -372,8 +287,19 @@ void CV_CharucoPoseEstimation::run(int) {
                                                 distCoeffs, rvec, tvec);
 
 
-                // check result
+                // check axes
+                const float offset = (board->getSquareLength() - board->getMarkerLength()) / 2.f;
+                vector<Point2f> axes = getAxis(cameraMatrix, distCoeffs, rvec, tvec, board->getSquareLength(), offset);
+                vector<Point2f> topLeft = getMarkerById(board->ids[0], corners, ids);
+                ASSERT_NEAR(topLeft[0].x, axes[1].x, 3.f);
+                ASSERT_NEAR(topLeft[0].y, axes[1].y, 3.f);
+                vector<Point2f> bottomLeft = getMarkerById(board->ids[2], corners, ids);
+                ASSERT_NEAR(bottomLeft[0].x, axes[2].x, 3.f);
+                ASSERT_NEAR(bottomLeft[0].y, axes[2].y, 3.f);
+
+                // check estimate result
                 vector< Point2f > projectedCharucoCorners;
+
                 projectPoints(board->chessboardCorners, rvec, tvec, cameraMatrix, distCoeffs,
                               projectedCharucoCorners);
 
@@ -434,9 +360,9 @@ void CV_CharucoDiamondDetection::run(int) {
 
     Mat distCoeffs(5, 1, CV_64FC1, Scalar::all(0));
     // for different perspectives
-    for(double distance = 0.3; distance <= 0.3; distance += 0.2) {
-        for(int yaw = 0; yaw < 360; yaw += 100) {
-            for(int pitch = 30; pitch <= 90; pitch += 30) {
+    for(double distance = 0.2; distance <= 0.3; distance += 0.1) {
+        for(int yaw = -50; yaw <= 50; yaw += 25) {
+            for(int pitch = -50; pitch <= 50; pitch += 25) {
 
                 int markerBorder = iter % 2 + 1;
                 for(int i = 0; i < 4; i++)
@@ -445,7 +371,7 @@ void CV_CharucoDiamondDetection::run(int) {
 
                 // get synthetic image
                 Mat rvec, tvec;
-                Mat img = projectCharucoBoard(board, cameraMatrix, deg2rad(pitch), deg2rad(yaw),
+                Mat img = projectCharucoBoard(board, cameraMatrix, deg2rad(yaw), deg2rad(pitch),
                                               distance, imgSize, markerBorder, rvec, tvec);
 
                 // detect markers
@@ -485,14 +411,21 @@ void CV_CharucoDiamondDetection::run(int) {
 
 
                 vector< Point2f > projectedDiamondCorners;
-                projectPoints(board->chessboardCorners, rvec, tvec, cameraMatrix, distCoeffs,
+
+                // copy chessboardCorners
+                vector<Point3f> copyChessboardCorners = board->chessboardCorners;
+                // move copyChessboardCorners points
+                for (size_t i = 0; i < copyChessboardCorners.size(); i++)
+                    copyChessboardCorners[i] -= board->rightBottomBorder / 2.f;
+
+                projectPoints(copyChessboardCorners, rvec, tvec, cameraMatrix, distCoeffs,
                               projectedDiamondCorners);
 
                 vector< Point2f > projectedDiamondCornersReorder(4);
-                projectedDiamondCornersReorder[0] = projectedDiamondCorners[2];
-                projectedDiamondCornersReorder[1] = projectedDiamondCorners[3];
-                projectedDiamondCornersReorder[2] = projectedDiamondCorners[1];
-                projectedDiamondCornersReorder[3] = projectedDiamondCorners[0];
+                projectedDiamondCornersReorder[0] = projectedDiamondCorners[0];
+                projectedDiamondCornersReorder[1] = projectedDiamondCorners[1];
+                projectedDiamondCornersReorder[2] = projectedDiamondCorners[3];
+                projectedDiamondCornersReorder[3] = projectedDiamondCorners[2];
 
 
                 for(unsigned int i = 0; i < 4; i++) {
@@ -506,18 +439,20 @@ void CV_CharucoDiamondDetection::run(int) {
                     }
                 }
 
+                Ptr<aruco::EstimateParameters> estimateParameters = aruco::EstimateParameters::create();
+                estimateParameters->pattern = aruco::CW_top_left_corner;
                 // estimate diamond pose
                 vector< Vec3d > estimatedRvec, estimatedTvec;
-                aruco::estimatePoseSingleMarkers(diamondCorners, squareLength, cameraMatrix,
-                                                 distCoeffs, estimatedRvec, estimatedTvec);
+                aruco::estimatePoseSingleMarkers(diamondCorners, squareLength, cameraMatrix, distCoeffs, estimatedRvec,
+                                                 estimatedTvec, noArray(), estimateParameters);
 
                 // check result
                 vector< Point2f > projectedDiamondCornersPose;
                 vector< Vec3f > diamondObjPoints(4);
-                diamondObjPoints[0] = Vec3f(-squareLength / 2.f, squareLength / 2.f, 0);
-                diamondObjPoints[1] = Vec3f(squareLength / 2.f, squareLength / 2.f, 0);
-                diamondObjPoints[2] = Vec3f(squareLength / 2.f, -squareLength / 2.f, 0);
-                diamondObjPoints[3] = Vec3f(-squareLength / 2.f, -squareLength / 2.f, 0);
+                diamondObjPoints[0] = Vec3f(0.f, 0.f, 0);
+                diamondObjPoints[1] = Vec3f(squareLength, 0.f, 0);
+                diamondObjPoints[2] = Vec3f(squareLength, squareLength, 0);
+                diamondObjPoints[3] = Vec3f(0.f, squareLength, 0);
                 projectPoints(diamondObjPoints, estimatedRvec[0], estimatedTvec[0], cameraMatrix,
                               distCoeffs, projectedDiamondCornersPose);
 
@@ -686,51 +621,198 @@ TEST(Charuco, testBoardSubpixelCoords)
         0, 0.5*res.height, 0.5*res.height,
         0, 0, 1);
 
-    // load board image with corners at round values
-    cv::String testImagePath = cvtest::TS::ptr()->get_data_path() + "aruco/" + "trivial_board_detection.png";
-    Mat img = imread(testImagePath);
+    // set expected_corners values
     cv::Mat expected_corners = (cv::Mat_<float>(9,2) <<
-        200, 300,
-        250, 300,
-        300, 300,
+        200, 200,
+        250, 200,
+        300, 200,
         200, 250,
         250, 250,
         300, 250,
-        200, 200,
-        250, 200,
-        300, 200
+        200, 300,
+        250, 300,
+        300, 300
     );
 
     cv::Mat gray;
-    cv::cvtColor(img, gray, cv::COLOR_BGR2GRAY);
 
     auto dict = cv::aruco::getPredefinedDictionary(cv::aruco::DICT_APRILTAG_36h11);
     auto board = cv::aruco::CharucoBoard::create(4, 4, 1.f, .8f, dict);
 
+    // generate ChArUco board
+    board->draw(Size(res.width, res.height), gray, 150);
+    cv::GaussianBlur(gray, gray, Size(5, 5), 1.0);
+
     auto params = cv::aruco::DetectorParameters::create();
     params->cornerRefinementMethod = cv::aruco::CORNER_REFINE_APRILTAG;
 
     std::vector<int> ids;
     std::vector<std::vector<cv::Point2f>> corners, rejected;
 
-    cv::aruco::detectMarkers(gray, dict, corners, ids, params, rejected, K);
+    cv::aruco::detectMarkers(gray, dict, corners, ids, params, rejected);
 
     ASSERT_EQ(ids.size(), size_t(8));
 
     cv::Mat c_ids, c_corners;
     cv::aruco::interpolateCornersCharuco(corners, ids, gray, board, c_corners, c_ids, K);
-    cv::Mat corners_reshaped = c_corners.reshape(1);
 
     ASSERT_EQ(c_corners.rows, expected_corners.rows);
-    EXPECT_NEAR(0, cvtest::norm(expected_corners, c_corners.reshape(1), NORM_INF), 1e-3);
+    EXPECT_NEAR(0, cvtest::norm(expected_corners, c_corners.reshape(1), NORM_INF), 1e-1);
 
     c_ids = cv::Mat();
     c_corners = cv::Mat();
     cv::aruco::interpolateCornersCharuco(corners, ids, gray, board, c_corners, c_ids);
-    corners_reshaped = c_corners.reshape(1);
 
     ASSERT_EQ(c_corners.rows, expected_corners.rows);
-    EXPECT_NEAR(0, cvtest::norm(expected_corners, c_corners.reshape(1), NORM_INF), 1e-3);
+    EXPECT_NEAR(0, cvtest::norm(expected_corners, c_corners.reshape(1), NORM_INF), 1e-1);
+}
+
+TEST(CV_ArucoTutorial, can_find_choriginal)
+{
+    string imgPath = cvtest::findDataFile("choriginal.jpg", false);
+    Mat image = imread(imgPath);
+    cv::Ptr<cv::aruco::Dictionary> dictionary = aruco::getPredefinedDictionary(aruco::DICT_6X6_250);
+    Ptr<aruco::DetectorParameters> detectorParams = aruco::DetectorParameters::create();
+
+    vector< int > ids;
+    vector< vector< Point2f > > corners, rejected;
+    const size_t N = 17ull;
+    // corners of aruco markers with indices goldCornersIds
+    const int goldCorners[N][8] = { {268,77,  290,80,  286,97,  263,94},  {360,90,  382,93,  379,111, 357,108},
+                                    {211,106, 233,109, 228,127, 205,123}, {306,120, 328,124, 325,142, 302,138},
+                                    {402,135, 425,139, 423,157, 400,154}, {247,152, 271,155, 267,174, 242,171},
+                                    {347,167, 371,171, 369,191, 344,187}, {185,185, 209,189, 203,210, 178,206},
+                                    {288,201, 313,206, 309,227, 284,223}, {393,218, 418,222, 416,245, 391,241},
+                                    {223,240, 250,244, 244,268, 217,263}, {333,258, 359,262, 356,286, 329,282},
+                                    {152,281, 179,285, 171,312, 143,307}, {267,300, 294,305, 289,331, 261,327},
+                                    {383,319, 410,324, 408,351, 380,347}, {194,347, 223,352, 216,382, 186,377},
+                                    {315,368, 345,373, 341,403, 310,398} };
+    map<int, const int*> mapGoldCorners;
+    for (int i = 0; i < static_cast<int>(N); i++)
+        mapGoldCorners[i] = goldCorners[i];
+
+    aruco::detectMarkers(image, dictionary, corners, ids, detectorParams, rejected);
+
+    ASSERT_EQ(N, ids.size());
+    for (size_t i = 0; i < N; i++)
+    {
+        int arucoId = ids[i];
+        ASSERT_EQ(4ull, corners[i].size());
+        ASSERT_TRUE(mapGoldCorners.find(arucoId) != mapGoldCorners.end());
+        for (int j = 0; j < 4; j++)
+        {
+            EXPECT_NEAR(static_cast<float>(mapGoldCorners[arucoId][j * 2]), corners[i][j].x, 1.f);
+            EXPECT_NEAR(static_cast<float>(mapGoldCorners[arucoId][j * 2 + 1]), corners[i][j].y, 1.f);
+        }
+    }
+}
+
+TEST(CV_ArucoTutorial, can_find_chocclusion)
+{
+    string imgPath = cvtest::findDataFile("chocclusion_original.jpg", false);
+    Mat image = imread(imgPath);
+    cv::Ptr<cv::aruco::Dictionary> dictionary = aruco::getPredefinedDictionary(aruco::DICT_6X6_250);
+    Ptr<aruco::DetectorParameters> detectorParams = aruco::DetectorParameters::create();
+
+    vector< int > ids;
+    vector< vector< Point2f > > corners, rejected;
+    const size_t N = 13ull;
+    // corners of aruco markers with indices goldCornersIds
+    const int goldCorners[N][8] = { {301,57, 322,62, 317,79, 295,73}, {391,80, 413,85, 408,103, 386,97},
+                                    {242,79, 264,85, 256,102, 234,96}, {334,103, 357,109, 352,126, 329,121},
+                                    {428,129, 451,134, 448,152, 425,146}, {274,128, 296,134, 290,153, 266,147},
+                                    {371,154, 394,160, 390,180, 366,174}, {208,155, 232,161, 223,181, 199,175},
+                                    {309,182, 333,188, 327,209, 302,203}, {411,210, 436,216, 432,238, 407,231},
+                                    {241,212, 267,219, 258,242, 232,235}, {167,244, 194,252, 183,277, 156,269},
+                                    {202,314, 230,322, 220,349, 191,341} };
+    map<int, const int*> mapGoldCorners;
+    const int goldCornersIds[N] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 15};
+    for (int i = 0; i < static_cast<int>(N); i++)
+        mapGoldCorners[goldCornersIds[i]] = goldCorners[i];
+
+    aruco::detectMarkers(image, dictionary, corners, ids, detectorParams, rejected);
+
+    ASSERT_EQ(N, ids.size());
+    for (size_t i = 0; i < N; i++)
+    {
+        int arucoId = ids[i];
+        ASSERT_EQ(4ull, corners[i].size());
+        ASSERT_TRUE(mapGoldCorners.find(arucoId) != mapGoldCorners.end());
+        for (int j = 0; j < 4; j++)
+        {
+            EXPECT_NEAR(static_cast<float>(mapGoldCorners[arucoId][j * 2]), corners[i][j].x, 1.f);
+            EXPECT_NEAR(static_cast<float>(mapGoldCorners[arucoId][j * 2 + 1]), corners[i][j].y, 1.f);
+        }
+    }
+}
+
+TEST(CV_ArucoTutorial, can_find_diamondmarkers)
+{
+    string imgPath = cvtest::findDataFile("diamondmarkers.png", false);
+    Mat image = imread(imgPath);
+
+    string dictPath = cvtest::findDataFile("tutorial_dict.yml", false);
+    Ptr<aruco::Dictionary> dictionary = makePtr<aruco::Dictionary>();
+    FileStorage fs(dictPath, FileStorage::READ);
+    dictionary->aruco::Dictionary::readDictionary(fs.root()); // set marker from tutorial_dict.yml
+
+    string detectorPath = cvtest::findDataFile("detector_params.yml", false);
+    fs = FileStorage(detectorPath, FileStorage::READ);
+    Ptr<aruco::DetectorParameters> detectorParams = aruco::DetectorParameters::create();
+    detectorParams->readDetectorParameters(fs.root());
+    detectorParams->cornerRefinementMethod = 3;
+
+    vector< int > ids;
+    vector< vector< Point2f > > corners, rejected;
+    const size_t N = 12ull;
+    // corner indices of ArUco markers
+    const int goldCornersIds[N] = { 4, 12, 11, 3, 12, 10, 12, 10, 10, 11, 2, 11 };
+    map<int, int> counterGoldCornersIds;
+    for (int i = 0; i < static_cast<int>(N); i++)
+        counterGoldCornersIds[goldCornersIds[i]]++;
+
+    aruco::detectMarkers(image, dictionary, corners, ids, detectorParams, rejected);
+    map<int, int> counterRes;
+    for (size_t i = 0; i < N; i++)
+    {
+        int arucoId = ids[i];
+        counterRes[arucoId]++;
+    }
+
+    ASSERT_EQ(N, ids.size());
+    EXPECT_EQ(counterGoldCornersIds, counterRes); // check the number of ArUco markers
+}
+
+TEST(Charuco, issue_14014)
+{
+    string imgPath = cvtest::findDataFile("aruco/recover.png");
+    Mat img = imread(imgPath);
+
+    Ptr<aruco::Dictionary> dict = aruco::getPredefinedDictionary(aruco::PREDEFINED_DICTIONARY_NAME(cv::aruco::DICT_7X7_250));
+    Ptr<aruco::CharucoBoard> board = aruco::CharucoBoard::create(8, 5, 0.03455f, 0.02164f, dict);
+    Ptr<aruco::DetectorParameters> detectorParams = aruco::DetectorParameters::create();
+    detectorParams->cornerRefinementMethod = aruco::CORNER_REFINE_SUBPIX;
+    detectorParams->cornerRefinementMinAccuracy = 0.01;
+
+    vector<Mat> corners, rejectedPoints;
+    vector<int> ids;
+
+    aruco::detectMarkers(img, dict, corners, ids, detectorParams, rejectedPoints);
+
+    ASSERT_EQ(corners.size(), 19ull);
+    EXPECT_EQ(Size(4, 1), corners[0].size()); // check dimension of detected corners
+
+    size_t numRejPoints = rejectedPoints.size();
+    ASSERT_EQ(rejectedPoints.size(), 26ull); // optional check to track regressions
+    EXPECT_EQ(Size(4, 1), rejectedPoints[0].size()); // check dimension of detected corners
+
+    aruco::refineDetectedMarkers(img, board, corners, ids, rejectedPoints);
+
+    ASSERT_EQ(corners.size(), 20ull);
+    EXPECT_EQ(Size(4, 1), corners[0].size()); // check dimension of rejected corners after successfully refine
+
+    ASSERT_EQ(rejectedPoints.size() + 1, numRejPoints);
+    EXPECT_EQ(Size(4, 1), rejectedPoints[0].size()); // check dimension of rejected corners after successfully refine
 }
 
 }} // namespace
