@@ -120,7 +120,7 @@ static void _threshold(cv::Mat& img, rlVec& res, double threshold, int type)
       _thresholdLine<double>((double*) img.ptr(i), img.cols, i, threshold, type, res);
     break;
   default:
-    CV_Error( CV_StsUnsupportedFormat, "unsupported image type" );
+    CV_Error( Error::StsUnsupportedFormat, "unsupported image type" );
   }
 }
 
@@ -207,7 +207,7 @@ void paint_impl(cv::Mat& img, rlType* pRuns, int nSize, T value)
         paint_impl<double>(_image, pRuns, N - 1, dValue);
         break;
     default:
-        CV_Error(CV_StsUnsupportedFormat, "unsupported image type");
+        CV_Error(Error::StsUnsupportedFormat, "unsupported image type");
         break;
     }
   }
@@ -801,7 +801,7 @@ CV_EXPORTS void morphologyEx(InputArray rlSrc, OutputArray rlDest, int op, Input
         break;
         default:
         case MORPH_HITMISS:
-            CV_Error(CV_StsBadArg, "unknown or unsupported morphological operation");
+            CV_Error(Error::StsBadArg, "unknown or unsupported morphological operation");
         }
         convertToOutputArray(runsDestination, sizeSource, rlDest);
     }
